# Generated by Django 3.0.6 on 2020-05-15 00:07

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0031_auto_20200422_2039"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalcoronakap",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalcoronakap",
            name="site",
        ),
        migrations.RemoveField(
            model_name="historicalcoronakap",
            name="subject_visit",
        ),
        migrations.AlterModelOptions(
            name="bloodresultsfbc",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: FBC",
                "verbose_name_plural": "Blood Results: FBC",
            },
        ),
        migrations.AlterModelOptions(
            name="bloodresultsglu",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: Glucose",
                "verbose_name_plural": "Blood Results: Glucose",
            },
        ),
        migrations.AlterModelOptions(
            name="bloodresultshba1c",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: HbA1c",
                "verbose_name_plural": "Blood Results: HbA1c",
            },
        ),
        migrations.AlterModelOptions(
            name="bloodresultslft",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: LFT",
                "verbose_name_plural": "Blood Results: LFT",
            },
        ),
        migrations.AlterModelOptions(
            name="bloodresultslipid",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: Lipids",
                "verbose_name_plural": "Blood Results: Lipids",
            },
        ),
        migrations.AlterModelOptions(
            name="bloodresultsrft",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Blood Result: RFT",
                "verbose_name_plural": "Blood Results: RFT",
            },
        ),
        migrations.AlterModelOptions(
            name="complications",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Presence of Complications",
                "verbose_name_plural": "Presence of Complications",
            },
        ),
        migrations.AlterModelOptions(
            name="followup",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Clinic follow up: Examination",
                "verbose_name_plural": "Clinic follow up: Examination",
            },
        ),
        migrations.AlterModelOptions(
            name="followupvitals",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Clinic follow up: Vitals",
                "verbose_name_plural": "Clinic follow ups: Vitals",
            },
        ),
        migrations.AlterModelOptions(
            name="healtheconomics",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Health Economics",
                "verbose_name_plural": "Health Economics",
            },
        ),
        migrations.AlterModelOptions(
            name="malariatest",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Malaria Test",
            },
        ),
        migrations.AlterModelOptions(
            name="medicationadherence",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Medication Adherence",
                "verbose_name_plural": "Medication Adherence",
            },
        ),
        migrations.AlterModelOptions(
            name="patienthistory",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Patient History",
                "verbose_name_plural": "Patient History",
            },
        ),
        migrations.AlterModelOptions(
            name="physicalexam",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Physical Exam",
                "verbose_name_plural": "Physical Exams",
            },
        ),
        migrations.AlterModelOptions(
            name="urinedipsticktest",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Urine Dipstick Test",
            },
        ),
        migrations.DeleteModel(
            name="CoronaKap",
        ),
        migrations.DeleteModel(
            name="HistoricalCoronaKap",
        ),
    ]



"""
A settings file with the bare minimum attributes.
"""

from pathlib import Path

import environ
from multisite import SiteID

APP_NAME = "meta_edc"
BASE_DIR = Path(__file__).resolve().parent.parent
ENV_DIR = Path(__file__).resolve().parent.parent

env = environ.Env()
env.read_env(ENV_DIR / ".env")

EDC_SITES_MODULE_NAME = env.str("EDC_SITES_MODULE_NAME")
ETC_DIR = env.str("DJANGO_ETC_FOLDER")
SECRET_KEY = "blahblahblah"  # noqa: S105
ALLOWED_HOSTS = ["*"]
SITE_ID = SiteID(default=1)  # 1 is not a site



# Generated by Django 5.0.7 on 2024-08-12 22:56

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0010_alter_patienthistorymissingbaselinecd4_options_and_more"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as created, \n    'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, \n    'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, 'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



from django.contrib import admin, messages
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.utils.translation import gettext_lazy as _
from django_audit_fields.admin import audit_fieldset_tuple
from edc_consent.actions import (
    flag_as_verified_against_paper,
    unflag_as_verified_against_paper,
)
from edc_identifier import SubjectIdentifierError, is_subject_identifier_or_raise
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.models import Medication
from edc_pharmacy.prescribe import create_prescription

from meta_edc.meta_version import get_meta_version
from meta_pharmacy.constants import METFORMIN
from meta_screening.models.subject_screening import SubjectScreening
from meta_subject.models import SubjectVisit

from ..forms import SubjectConsentForm
from ..models import SubjectConsent


class SubjectConsentModelAdminMixin:
    form = SubjectConsentForm

    actions = (
        flag_as_verified_against_paper,
        unflag_as_verified_against_paper,
        "create_missing_metformin_rx",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "screening_identifier",
                    "subject_identifier",
                    "first_name",
                    "last_name",
                    "initials",
                    "gender",
                    "language",
                    "is_literate",
                    "witness_name",
                    "consent_datetime",
                    "dob",
                    "guardian_name",
                    "is_dob_estimated",
                    "identity",
                    "identity_type",
                    "confirm_identity",
                    "is_incarcerated",
                )
            },
        ),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": _("The following questions are directed to the interviewer."),
            },
        ),
        audit_fieldset_tuple,
    )

    search_fields = ("subject_identifier", "screening_identifier", "identity")

    radio_fields = {  # noqa: RUF012
        "gender": admin.VERTICAL,
        "assessment_score": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "is_dob_estimated": admin.VERTICAL,
        "identity_type": admin.VERTICAL,
        "is_incarcerated": admin.VERTICAL,
        "is_literate": admin.VERTICAL,
        "language": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
    }

    def delete_view(self, request, object_id, extra_context=None):
        """Prevent deletion if SubjectVisit objects exist."""
        extra_context = extra_context or {}
        obj = SubjectConsent.objects.get(id=object_id)
        try:
            protected = [SubjectVisit.objects.get(subject_identifier=obj.subject_identifier)]
        except ObjectDoesNotExist:
            protected = None
        except MultipleObjectsReturned:
            protected = SubjectVisit.objects.filter(subject_identifier=obj.subject_identifier)
        extra_context.update({"protected": protected})
        return super().delete_view(request, object_id, extra_context)

    def get_next_options(self, request=None, **kwargs):
        """Returns the key/value pairs from the "next" querystring
        as a dictionary.
        """
        next_options = super().get_next_options(request=request, **kwargs)
        try:
            is_subject_identifier_or_raise(next_options["subject_identifier"])
        except SubjectIdentifierError:
            next_options["subject_identifier"] = SubjectScreening.objects.get(
                subject_identifier_as_pk=next_options["subject_identifier"]
            ).subject_identifier
        except KeyError:
            pass
        return next_options

    @admin.action(permissions=["view"], description=_("Create missing METFORMIN prescription"))
    def create_missing_metformin_rx(self, request, queryset):
        medication = Medication.objects.get(name=METFORMIN)
        total = queryset.count()
        created = 0
        exist = 0
        for instance in queryset:
            try:
                create_prescription(
                    subject_identifier=instance.subject_identifier,
                    report_datetime=instance.consent_datetime,
                    randomizer_name=get_meta_version(),
                    medication_names=[medication.name],
                    site_id=instance.site.id,
                )
            except PrescriptionAlreadyExists:
                exist += 1
            else:
                created += 1
        messages.success(
            request,
            _(
                "Created %(created)s/%(total)s missing %(display_name)s prescriptions. "
                "Got %(exist)s/%(total)s prescriptions already exist"
            )
            % {
                "created": created,
                "exist": exist,
                "total": total,
                "display_name": medication.display_name,
            },
        )



# Generated by Django 6.0 on 2025-01-28 23:32

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_facility", "0014_healthfacility_title_historicalhealthfacility_title"),
        ("meta_subject", "0216_historicalnextappointment_nextappointment"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalnextappointment",
            name="appt_datetime",
            field=models.DateTimeField(
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment date and time",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="appt_datetime",
            field=models.DateTimeField(
                help_text="Should fall on an valid clinic day for this facility",
                null=True,
                verbose_name="Next scheduled routine/facility appointment date and time",
            ),
        ),
        migrations.AlterField(
            model_name="nextappointment",
            name="health_facility",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_facility.healthfacility",
            ),
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-21 22:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0023_endpoints_meta_report_subject_a56b22_idx"),
    ]

    operations = [
        migrations.CreateModel(
            name="GlucoseSummary",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("fbg_value", models.DecimalField(decimal_places=2, max_digits=8, null=True)),
                ("fbg_datetime", models.DateTimeField(null=True)),
                ("ogtt_value", models.DecimalField(decimal_places=2, max_digits=8, null=True)),
                ("ogtt_datetime", models.DateTimeField(null=True)),
            ],
            options={
                "verbose_name": "GlucoseSummary",
                "verbose_name_plural": "GlucoseSummary",
                "db_table": "glucose_summary_view",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
    ]



# Generated by Django 4.2.10 on 2024-03-08 02:14

from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0018_missedreferralreasons"),
        ("meta_subject", "0179_alter_birthoutcomes_consent_model_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="missed_referral_reasons",
            field=models.ManyToManyField(
                blank=True,
                to="meta_lists.missedreferralreasons",
                verbose_name="If 'No', please provide a reason for not seeking further care or follow up?",
            ),
        ),
        migrations.AddField(
            model_name="dmreferralfollowup",
            name="other_missed_referral_reason",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other 'reason for not seeking further care', please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreferralfollowup",
            name="other_missed_referral_reason",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other 'reason for not seeking further care', please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="dmreferralfollowup",
            name="not_attended_reason",
            field=models.TextField(
                default="QUESTION_RETIRED",
                editable=False,
                verbose_name="If 'No', please provide a reason for not seeking further care or follow up?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferralfollowup",
            name="not_attended_reason",
            field=models.TextField(
                default="QUESTION_RETIRED",
                editable=False,
                verbose_name="If 'No', please provide a reason for not seeking further care or follow up?",
            ),
        ),
    ]



from .subject_screening_button import (
    SubjectScreeningButton,
    SubjectScreeningPartOneButton,
    SubjectScreeningPartThreeButton,
    SubjectScreeningPartTwoButton,
)

__all__ = [
    "SubjectScreeningButton",
    "SubjectScreeningPartOneButton",
    "SubjectScreeningPartThreeButton",
    "SubjectScreeningPartTwoButton",
]



# Generated by Django 5.0.8 on 2024-08-24 01:12

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0039_onstudymissingvalues"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.glucose_summary_view' as `report_model` from (SELECT v.subject_identifier, fbg_value, fbg_datetime, NULL AS `ogtt_value`, NULL AS `ogtt_datetime`, CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS `fasted`, fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucosefbg AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier UNION SELECT v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime, CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS `fasted`, fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucose AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nselect *, uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as \'ogtt_value\', null as \'ogtt_datetime\',\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as \'fasted\',\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'meta_reports.glucose_summary_view' as report_model from (SELECT v.subject_identifier, fbg_value, fbg_datetime, NULL AS \"ogtt_value\", NULL AS \"ogtt_datetime\", CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS \"fasted\", fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucosefbg AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier UNION SELECT v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime, CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS \"fasted\", fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucose AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nselect *, get_random_uuid() as id, now() as created, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'meta_reports.glucose_summary_view' as report_model from (SELECT v.subject_identifier, fbg_value, fbg_datetime, NULL AS \"ogtt_value\", NULL AS \"ogtt_datetime\", CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS \"fasted\", fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucosefbg AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier UNION SELECT v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime, CASE WHEN fasting = 'fasting' THEN 'Yes' WHEN fasting = 'non_fasting' THEN 'No' ELSE fasting END AS \"fasted\", fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime FROM meta_subject_glucose AS fbg LEFT JOIN meta_subject_subjectvisit AS v ON v.id = fbg.subject_visit_id LEFT JOIN meta_prn_endofstudy AS eos ON v.subject_identifier = eos.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                '# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || \'-\' || hex(randomblob(2)) || \'-\' || \'4\' ||\n    substr(hex( randomblob(2)), 2) || \'-\' ||\n    substr(\'AB89\', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || \'-\' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, \'meta_reports.glucose_summary_view\' as report_model\nfrom (\n    select v.subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucosefbg as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier\n    UNION\n    select v.subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    case when fasting="fasting" then "Yes" when fasting="non_fasting" then "No" else fasting end as fasted,\n    fbg.site_id, v.visit_code, v.visit_code_sequence, v.appointment_id, eos.offstudy_datetime\n    from meta_subject_glucose as fbg\n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    left join meta_prn_endofstudy as eos on v.subject_identifier=eos.subject_identifier    \n    ) as A\norder by subject_identifier, fbg_datetime',
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.missing_screening_ogtt_view' as `report_model` from (SELECT screening_identifier, site_id, report_datetime AS `screening_datetime`, fbg_datetime, converted_fbg_value AS `fbg_value`, converted_ogtt_value AS `ogtt_value`, fbg2_value, ogtt2_value, repeat_glucose_performed AS `repeated`, p3_ltfu, fbg2_datetime, ogtt2_datetime, consented, screening_identifier AS `subject_identifier`, id AS `original_id` FROM meta_screening_subjectscreening WHERE NOT converted_fbg_value IS NULL AND converted_ogtt_value IS NULL AND unsuitable_agreed <> 'Yes') as A ORDER BY subject_identifier, site_id",
                "missing_screening_ogtt_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, uuid() as id, now() as created,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, repeat_glucose_performed as repeated,\n        p3_ltfu, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented,\n        screening_identifier as subject_identifier, id as original_id\n        from meta_screening_subjectscreening\n        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != \"Yes\"\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, get_random_uuid() as id, now() as created, \'meta_reports.missing_screening_ogtt_view\' as report_model from (SELECT screening_identifier, site_id, report_datetime AS "screening_datetime", fbg_datetime, converted_fbg_value AS "fbg_value", converted_ogtt_value AS "ogtt_value", fbg2_value, ogtt2_value, repeat_glucose_performed AS "repeated", p3_ltfu, fbg2_datetime, ogtt2_datetime, consented, screening_identifier AS "subject_identifier", id AS "original_id" FROM meta_screening_subjectscreening WHERE NOT converted_fbg_value IS NULL AND converted_ogtt_value IS NULL AND unsuitable_agreed <> \'Yes\') as A ORDER BY subject_identifier, site_id',
                "missing_screening_ogtt_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, get_random_uuid() as id, now() as created,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, repeat_glucose_performed as repeated,\n        p3_ltfu, fbg2_value, ogtt2_value, fbg2_datetime, ogtt2_datetime, consented,\n        screening_identifier as subject_identifier, id as original_id\n        from meta_screening_subjectscreening\n        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != \"Yes\"\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, uuid() as id, datetime() as created, \'meta_reports.missing_screening_ogtt_view\' as report_model from (SELECT screening_identifier, site_id, report_datetime AS "screening_datetime", fbg_datetime, converted_fbg_value AS "fbg_value", converted_ogtt_value AS "ogtt_value", fbg2_value, ogtt2_value, repeat_glucose_performed AS "repeated", p3_ltfu, fbg2_datetime, ogtt2_datetime, consented, screening_identifier AS "subject_identifier", id AS "original_id" FROM meta_screening_subjectscreening WHERE NOT converted_fbg_value IS NULL AND converted_ogtt_value IS NULL AND unsuitable_agreed <> \'Yes\') as A ORDER BY subject_identifier, site_id',
                "missing_screening_ogtt_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`,\n    'meta_reports.missing_screening_ogtt_view' as report_model\n      from (\n        select screening_identifier, site_id, report_datetime as 'screening_datetime', fbg_datetime,\n        converted_fbg_value as fbg_value,  converted_ogtt_value as ogtt_value, fbg2_value, ogtt2_value,\n        repeat_glucose_performed as repeated, p3_ltfu, fbg2_datetime, ogtt2_datetime, consented,\n        screening_identifier as subject_identifier, id as original_id\n        from meta_screening_subjectscreening\n        where converted_fbg_value is not null and converted_ogtt_value is null and unsuitable_agreed != \"Yes\"\n    ) as A\n    order by screening_identifier",
                "missing_screening_ogtt_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'onstudy_missing_values_view' as `report_model` from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'YES' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "onstudy_missing_values_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'YES' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "onstudy_missing_values_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'onstudy_missing_values_view' as report_model from (SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No HIV Diagnosis date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.hiv_diagnosis_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No VL date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.viral_load_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 value' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4 IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No CD4 date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.cd4_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'No current ARV start date but previous ARV is YES' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE crf.has_previous_arv_regimen = 'YES' AND crf.current_arv_regimen_start_date IS NULL GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified UNION SELECT v.subject_identifier, crf.id AS original_id, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified, 'meta_subject.patienthistory' AS label_lower, 'Other current ARV regimen missing' AS label, COUNT(*) AS records FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON v.id = crf.subject_visit_id LEFT JOIN meta_lists_arvregimens AS arvregimen ON crf.current_arv_regimen_id = arvregimen.id WHERE crf.other_current_arv_regimen IS NULL AND arvregimen.name = 'OTHER' GROUP BY v.subject_identifier, crf.subject_visit_id, crf.report_datetime, crf.site_id, v.visit_code, v.visit_code_sequence, v.schedule_name, crf.modified) as A ORDER BY subject_identifier, site_id",
                "onstudy_missing_values_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "onstudy_missing_values_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.patienthistorymissingbaselinecd4' as `report_model` from (SELECT subject_identifier, v.visit_code, v.visit_code_sequence, cd4, cd4_date, crf.site_id, crf.user_created, crf.user_modified, crf.modified FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON crf.subject_visit_id = v.id WHERE cd4 IS NULL OR cd4_date IS NULL) as A ORDER BY subject_identifier, site_id",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, now() as created, \n    'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'meta_reports.patienthistorymissingbaselinecd4' as report_model from (SELECT subject_identifier, v.visit_code, v.visit_code_sequence, cd4, cd4_date, crf.site_id, crf.user_created, crf.user_modified, crf.modified FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON crf.subject_visit_id = v.id WHERE cd4 IS NULL OR cd4_date IS NULL) as A ORDER BY subject_identifier, site_id",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, \n    'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'meta_reports.patienthistorymissingbaselinecd4' as report_model from (SELECT subject_identifier, v.visit_code, v.visit_code_sequence, cd4, cd4_date, crf.site_id, crf.user_created, crf.user_modified, crf.modified FROM meta_subject_patienthistory AS crf LEFT JOIN meta_subject_subjectvisit AS v ON crf.subject_visit_id = v.id WHERE cd4 IS NULL OR cd4_date IS NULL) as A ORDER BY subject_identifier, site_id",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "SELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, 'meta_reports.patienthistorymissingbaselinecd4' as report_model \n      from (\n        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,\n          cd4_date, crf.site_id, crf.user_created, crf.user_modified,\n          crf.modified\n        from meta_subject_patienthistory as crf\n        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id\n        where cd4 is null or cd4_date is null\n    ) as A\n    order by subject_identifier",
                "patient_history_missing_baseline_cd4_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.unattendedthreeinrow' as `report_model` from (SELECT subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, `third_value`, DATEDIFF(`third_date`, `first_date`) AS `interval_days`, DATEDIFF(NOW(), `first_date`) AS `from_now_days` FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS `first_value`, NTH_VALUE(visit_code, 2) OVER w AS `second_value`, NTH_VALUE(visit_code, 3) OVER w AS `third_value`, FIRST_VALUE(appt_datetime) OVER w AS `first_date`, NTH_VALUE(appt_datetime, 3) OVER w AS `third_date` FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = 'New' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT `second_value` IS NULL AND NOT `third_value` IS NULL) as A ORDER BY subject_identifier, site_id",
                "unattended_three_in_row_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow' as `report_model` from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as `first_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `third_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `first_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `third_date`\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= now()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where `second_value` is not null and `third_value` is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, get_random_uuid() as id, now() as created, \'meta_reports.unattendedthreeinrow\' as report_model from (SELECT subject_identifier, site_id, appt_datetime, "first_value", "second_value", "third_value", CAST(AGE(CAST("third_date" AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS "interval_days", CAST(AGE(CAST(NOW() AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS "from_now_days" FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS "first_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "third_value", FIRST_VALUE(appt_datetime) OVER w AS "first_date", NTH_VALUE(appt_datetime, 3) OVER w AS "third_date" FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = \'New\' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime NULLS FIRST ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND NOT "third_value" IS NULL) as A ORDER BY subject_identifier, site_id',
                "unattended_three_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, gen_random_uuid() as id, now() as created,\n    'meta_reports.unattendedthreeinrow' as report_model\n    from (\n    select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n    EXTRACT(DAY FROM third_date - first_date) as interval_days,\n    EXTRACT(DAY FROM now() - first_date) as from_now_days\n    from (\n    select subject_identifier,site_id,appt_datetime,\n    FIRST_VALUE(visit_code) OVER w as first_value,\n    NTH_VALUE(visit_code, 2) OVER w as second_value,\n    NTH_VALUE(visit_code, 3) OVER w as third_value,\n    FIRST_VALUE(appt_datetime) OVER w as first_date,\n    NTH_VALUE(appt_datetime, 3) OVER w as third_date\n    from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n    and appt_datetime <= now()\n    WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n    ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, uuid() as id, datetime() as created, \'meta_reports.unattendedthreeinrow\' as report_model from (SELECT subject_identifier, site_id, appt_datetime, "first_value", "second_value", "third_value", CAST((JULIANDAY("third_date") - JULIANDAY("first_date")) AS INTEGER) AS "interval_days", CAST((JULIANDAY(NOW()) - JULIANDAY("first_date")) AS INTEGER) AS "from_now_days" FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS "first_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "third_value", FIRST_VALUE(appt_datetime) OVER w AS "first_date", NTH_VALUE(appt_datetime, 3) OVER w AS "third_date" FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = \'New\' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND NOT "third_value" IS NULL) as A ORDER BY subject_identifier, site_id',
                "unattended_three_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created,'meta_reports.unattendedthreeinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS INTEGER) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS INTEGER) as from_now_days\n        from (\n            select subject_identifier, site_id, appt_datetime,\n            FIRST_VALUE(visit_code) OVER w as first_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as third_value,\n            FIRST_VALUE(appt_datetime) OVER w as first_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as third_date\n            from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n            and appt_datetime <= datetime()\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n        ) as A\n    where second_value is not null and third_value is not null\n    ) as B\n    order by site_id, from_now_days desc",
                "unattended_three_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "with appointments as (select subject_identifier, site_id, visit_code, visit_code_sequence,\nappt_datetime, case when appt_timing=\"missed\" then \"New\" else appt_status end as `appt_status`,\ncase when appt_timing=\"missed\" then 1 else 0 end as `missed`\nfrom edc_appointment_appointment\nwhere visit_code_sequence=0 and appt_datetime<=now()\norder by subject_identifier, appt_datetime) select *, uuid() as id, now() as `created`, 'meta_reports.unattendedthreeinrow2' as `report_model` from (SELECT DISTINCT subject_identifier, site_id, `first_value`, `second_value`, `third_value`, DATEDIFF(`third_date`, `first_date`) AS interval_days, DATEDIFF(NOW(), `first_date`) AS from_now_days, `first_status`, `second_status`, `third_status`, SUM(missed) AS missed_count FROM (SELECT subject_identifier, site_id, appt_datetime, `missed`, FIRST_VALUE(appt_status) OVER w AS `third_status`, NTH_VALUE(appt_status, 2) OVER w AS `second_status`, NTH_VALUE(appt_status, 3) OVER w AS `first_status`, FIRST_VALUE(visit_code) OVER w AS `third_value`, NTH_VALUE(visit_code, 2) OVER w AS `second_value`, NTH_VALUE(visit_code, 3) OVER w AS `first_value`, FIRST_VALUE(appt_datetime) OVER w AS `third_date`, NTH_VALUE(appt_datetime, 2) OVER w AS `second_date`, NTH_VALUE(appt_datetime, 3) OVER w AS `first_date` FROM appointments WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime DESC ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT `second_value` IS NULL AND NOT `third_value` IS NULL AND `first_status` = 'New' AND `second_status` = 'New' AND `third_status` = 'New' GROUP BY subject_identifier, site_id, `first_value`, `second_value`, `third_value`, DATEDIFF(`third_date`, `first_date`), DATEDIFF(NOW(), `first_date`), `first_status`, `second_status`, `third_status` ORDER BY subject_identifier, site_id) as A ORDER BY subject_identifier, site_id",
                "unattended_three_in_row2_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedthreeinrow2' as `report_model` from (\n        select  distinct subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n            datediff(third_date, first_date) as `interval_days`,\n            datediff(now(), first_date) as `from_now_days`,\n            `first_status`, `second_status`, `third_status`, sum(missed) as `missed_count`\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as `third_status`,\n            NTH_VALUE(appt_status, 2) OVER w as `second_status`,\n            NTH_VALUE(appt_status, 3) OVER w as `first_status`,\n            FIRST_VALUE(visit_code) OVER w as `third_value`,\n            NTH_VALUE(visit_code, 2) OVER w as `second_value`,\n            NTH_VALUE(visit_code, 3) OVER w as `first_value`,\n            FIRST_VALUE(appt_datetime) OVER w as `third_date`,\n            NTH_VALUE(appt_datetime, 2) OVER w as `second_date`,\n            NTH_VALUE(appt_datetime, 3) OVER w as `first_date`\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where `second_value` is not null and `third_value` is not null\n          and `first_status`='New'\n          and `second_status`='New'\n          and `third_status`='New'\n        group by subject_identifier, site_id,  `first_value`, `second_value`, `third_value`,\n                 datediff(`third_date`, `first_date`),\n                 datediff(now(), `first_date`),\n                 `first_status`, `second_status`, `third_status`\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'with appointments as (select subject_identifier, site_id, visit_code, visit_code_sequence,\nappt_datetime, case when appt_timing="missed" then "New" else appt_status end as `appt_status`,\ncase when appt_timing="missed" then 1 else 0 end as `missed`\nfrom edc_appointment_appointment\nwhere visit_code_sequence=0 and appt_datetime<=now()\norder by subject_identifier, appt_datetime) select *, get_random_uuid() as id, now() as created, \'meta_reports.unattendedthreeinrow2\' as report_model from (SELECT DISTINCT subject_identifier, site_id, "first_value", "second_value", "third_value", CAST(AGE(CAST("third_date" AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS interval_days, CAST(AGE(CAST(NOW() AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS from_now_days, "first_status", "second_status", "third_status", SUM(missed) AS missed_count FROM (SELECT subject_identifier, site_id, appt_datetime, "missed", FIRST_VALUE(appt_status) OVER w AS "third_status", NTH_VALUE(appt_status, 2) OVER w AS "second_status", NTH_VALUE(appt_status, 3) OVER w AS "first_status", FIRST_VALUE(visit_code) OVER w AS "third_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "first_value", FIRST_VALUE(appt_datetime) OVER w AS "third_date", NTH_VALUE(appt_datetime, 2) OVER w AS "second_date", NTH_VALUE(appt_datetime, 3) OVER w AS "first_date" FROM appointments WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime DESC NULLS LAST ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND NOT "third_value" IS NULL AND "first_status" = \'New\' AND "second_status" = \'New\' AND "third_status" = \'New\' GROUP BY subject_identifier, site_id, "first_value", "second_value", "third_value", CAST(AGE(CAST("third_date" AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT), CAST(AGE(CAST(NOW() AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT), "first_status", "second_status", "third_status" ORDER BY subject_identifier NULLS FIRST, site_id NULLS FIRST) as A ORDER BY subject_identifier, site_id',
                "unattended_three_in_row2_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as appt_status,\n        case when appt_timing='missed' then 1 else 0 end as missed\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedthreeinrow2' as report_model\n    from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            EXTRACT(DAY FROM third_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 EXTRACT(DAY FROM third_date - first_date),\n                 EXTRACT(DAY FROM now() - first_date),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'with appointments as (select subject_identifier, site_id, visit_code, visit_code_sequence,\nappt_datetime, case when appt_timing="missed" then "New" else appt_status end as `appt_status`,\ncase when appt_timing="missed" then 1 else 0 end as `missed`\nfrom edc_appointment_appointment\nwhere visit_code_sequence=0 and appt_datetime<=now()\norder by subject_identifier, appt_datetime) select *, uuid() as id, datetime() as created, \'meta_reports.unattendedthreeinrow2\' as report_model from (SELECT DISTINCT subject_identifier, site_id, "first_value", "second_value", "third_value", CAST((JULIANDAY("third_date") - JULIANDAY("first_date")) AS INTEGER) AS interval_days, CAST((JULIANDAY(NOW()) - JULIANDAY("first_date")) AS INTEGER) AS from_now_days, "first_status", "second_status", "third_status", SUM(missed) AS missed_count FROM (SELECT subject_identifier, site_id, appt_datetime, "missed", FIRST_VALUE(appt_status) OVER w AS "third_status", NTH_VALUE(appt_status, 2) OVER w AS "second_status", NTH_VALUE(appt_status, 3) OVER w AS "first_status", FIRST_VALUE(visit_code) OVER w AS "third_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "first_value", FIRST_VALUE(appt_datetime) OVER w AS "third_date", NTH_VALUE(appt_datetime, 2) OVER w AS "second_date", NTH_VALUE(appt_datetime, 3) OVER w AS "first_date" FROM appointments WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime DESC ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND NOT "third_value" IS NULL AND "first_status" = \'New\' AND "second_status" = \'New\' AND "third_status" = \'New\' GROUP BY subject_identifier, site_id, "first_value", "second_value", "third_value", CAST((JULIANDAY("third_date") - JULIANDAY("first_date")) AS INTEGER), CAST((JULIANDAY(NOW()) - JULIANDAY("first_date")) AS INTEGER), "first_status", "second_status", "third_status" ORDER BY subject_identifier, site_id) as A ORDER BY subject_identifier, site_id',
                "unattended_three_in_row2_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    with appointments as (\n        select subject_identifier, site_id, visit_code, visit_code_sequence, appt_datetime,\n        case when appt_timing='missed' then 'New' else appt_status end as `appt_status`,\n        case when appt_timing='missed' then 1 else 0 end as `missed`\n        from edc_appointment_appointment\n        where visit_code_sequence=0 and appt_datetime<=now()\n        order by subject_identifier, appt_datetime\n        )\n    SELECT *, lower(\n            hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n            substr(hex( randomblob(2)), 2) || '-' ||\n            substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n            substr(hex(randomblob(2)), 2) || '-' ||\n            hex(randomblob(6))\n          ) as id, datetime() as created, 'meta_reports.unattendedthreeinrow2' as report_model from (\n        select  distinct subject_identifier, site_id,  first_value, second_value, third_value,\n            CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days,\n            first_status, second_status, third_status, sum(missed) as missed_count\n        from (\n            select subject_identifier,site_id,appt_datetime, missed,\n            FIRST_VALUE(appt_status) OVER w as third_status,\n            NTH_VALUE(appt_status, 2) OVER w as second_status,\n            NTH_VALUE(appt_status, 3) OVER w as first_status,\n            FIRST_VALUE(visit_code) OVER w as third_value,\n            NTH_VALUE(visit_code, 2) OVER w as second_value,\n            NTH_VALUE(visit_code, 3) OVER w as first_value,\n            FIRST_VALUE(appt_datetime) OVER w as third_date,\n            NTH_VALUE(appt_datetime, 2) OVER w as second_date,\n            NTH_VALUE(appt_datetime, 3) OVER w as first_date\n            from appointments\n            WINDOW w as (PARTITION BY subject_identifier order by appt_datetime desc ROWS UNBOUNDED PRECEDING)\n        ) as A\n        where second_value is not null and third_value is not null\n          and first_status='New'\n          and second_status='New'\n          and third_status='New'\n        group by subject_identifier, site_id,  first_value, second_value, third_value,\n                 CAST(JulianDay(third_date) - JulianDay(first_date) AS Integer),\n                 CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer),\n                 first_status, second_status, third_status\n        order by subject_identifier, site_id\n    ) as B",
                "unattended_three_in_row2_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.unattendedtwoinrow' as `report_model` from (SELECT subject_identifier, site_id, appt_datetime, `first_value`, `second_value`, DATEDIFF(`second_date`, `first_date`) AS interval_days, DATEDIFF(NOW(), `first_date`) AS from_now_days FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS `first_value`, NTH_VALUE(visit_code, 2) OVER w AS `second_value`, NTH_VALUE(visit_code, 3) OVER w AS `third_value`, FIRST_VALUE(appt_datetime) OVER w AS `first_date`, NTH_VALUE(appt_datetime, 2) OVER w AS `second_date` FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = 'New' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT `second_value` IS NULL AND `third_value` IS NULL) as A ORDER BY site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, uuid() as `id`, now() as `created`, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, `first_value`, `second_value`,\n            datediff(`second_date`, `first_date`) as `interval_days`,\n            datediff(now(), `first_date`) as `from_now_days`\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(`visit_code`) OVER w as `first_value`,\n                NTH_VALUE(`visit_code`, 2) OVER w as `second_value`,\n                NTH_VALUE(`visit_code`, 3) OVER w as `third_value`,\n                FIRST_VALUE(`appt_datetime`) OVER w as `first_date`,\n                NTH_VALUE(`appt_datetime`, 2) OVER w as `second_date`\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where `second_value` is not null and `third_value` is null\n    ) as A2\n    order by site_id, `from_now_days` desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, get_random_uuid() as id, now() as created, \'meta_reports.unattendedtwoinrow\' as report_model from (SELECT subject_identifier, site_id, appt_datetime, "first_value", "second_value", CAST(AGE(CAST("second_date" AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS interval_days, CAST(AGE(CAST(NOW() AS TIMESTAMP), CAST("first_date" AS TIMESTAMP)) AS BIGINT) AS from_now_days FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS "first_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "third_value", FIRST_VALUE(appt_datetime) OVER w AS "first_date", NTH_VALUE(appt_datetime, 2) OVER w AS "second_date" FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = \'New\' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime NULLS FIRST ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND "third_value" IS NULL) as A ORDER BY site_id, from_now_days desc',
                "unattended_two_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\n    select *, gen_random_uuid() as id, now() as created, 'meta_reports.unattendedtwoinrow' as report_model\n    from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            EXTRACT(DAY FROM second_date - first_date) as interval_days,\n            EXTRACT(DAY FROM now() - first_date) as from_now_days\n            from (\n                select subject_identifier, site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= now()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                'select *, uuid() as id, datetime() as created, \'meta_reports.unattendedtwoinrow\' as report_model from (SELECT subject_identifier, site_id, appt_datetime, "first_value", "second_value", CAST((JULIANDAY("second_date") - JULIANDAY("first_date")) AS INTEGER) AS interval_days, CAST((JULIANDAY(NOW()) - JULIANDAY("first_date")) AS INTEGER) AS from_now_days FROM (SELECT subject_identifier, site_id, appt_datetime, FIRST_VALUE(visit_code) OVER w AS "first_value", NTH_VALUE(visit_code, 2) OVER w AS "second_value", NTH_VALUE(visit_code, 3) OVER w AS "third_value", FIRST_VALUE(appt_datetime) OVER w AS "first_date", NTH_VALUE(appt_datetime, 2) OVER w AS "second_date" FROM edc_appointment_appointment WHERE visit_code_sequence = 0 AND appt_status = \'New\' AND appt_datetime <= NOW() WINDOW w AS (PARTITION BY subject_identifier ORDER BY appt_datetime ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW)) AS B WHERE NOT "second_value" IS NULL AND "third_value" IS NULL) as A ORDER BY site_id, from_now_days desc',
                "unattended_two_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as created, 'meta_reports.unattendedtwoinrow' as report_model from (\n        select subject_identifier, site_id, appt_datetime, first_value, second_value,\n            CAST(JulianDay(second_date) - JulianDay(first_date) AS Integer) as interval_days,\n            CAST(JulianDay(datetime()) - JulianDay(first_date) AS Integer) as from_now_days\n            from (\n                select subject_identifier ,site_id, appt_datetime,\n                FIRST_VALUE(visit_code) OVER w as first_value,\n                NTH_VALUE(visit_code, 2) OVER w as second_value,\n                NTH_VALUE(visit_code, 3) OVER w as third_value,\n                FIRST_VALUE(appt_datetime) OVER w as first_date,\n                NTH_VALUE(appt_datetime, 2) OVER w as second_date\n                from edc_appointment_appointment where visit_code_sequence=0 and appt_status='New'\n                and appt_datetime <= datetime()\n                WINDOW w as (PARTITION BY subject_identifier order by appt_datetime ROWS UNBOUNDED PRECEDING)\n            ) as A1\n        where second_value is not null and third_value is null\n    ) as A2\n    order by site_id, from_now_days desc",
                "unattended_two_in_row_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



import contextlib

from clinicedc_constants import YES
from django.core.exceptions import ValidationError
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_action_item.delete_action_item import ActionItemDeleteError, delete_action_item
from edc_pharmacy.exceptions import PrescriptionAlreadyExists
from edc_pharmacy.prescribe import create_prescription
from edc_randomization.site_randomizers import site_randomizers
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from meta_edc.meta_version import get_meta_version
from meta_screening.models import SubjectScreening
from meta_subject.models import SubjectVisit

from ..action_items import ReconsentAction
from .subject_consent import SubjectConsent
from .subject_consent_v1 import SubjectConsentV1


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsentV1,
    dispatch_uid="subject_consent_on_post_save",
)
def subject_consent_on_post_save(sender, instance, raw, created, **kwargs):  # noqa: ARG001
    """Creates an onschedule instance for this consented subject, if
    it does not exist.
    """

    if not raw:
        if not created:
            _, schedule = site_visit_schedules.get_by_onschedule_model("meta_prn.onschedule")
            schedule.refresh_schedule(instance.subject_identifier)
        else:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=instance.screening_identifier
            )
            if not subject_screening.consented:
                subject_screening.subject_identifier = instance.subject_identifier
                subject_screening.consented = True
                subject_screening.save_base(update_fields=["subject_identifier", "consented"])

                # randomize
                site_randomizers.randomize(
                    get_meta_version(),
                    identifier=instance.subject_identifier,
                    report_datetime=instance.consent_datetime,
                    site=instance.site,
                    user=instance.user_created,
                    gender=instance.gender,
                )

                # put subject on schedule
                _, schedule = site_visit_schedules.get_by_onschedule_model(
                    "meta_prn.onschedule"
                )
                schedule.put_on_schedule(
                    subject_identifier=instance.subject_identifier,
                    onschedule_datetime=instance.consent_datetime,
                )
                # All refills are created against this prescription
                with contextlib.suppress(PrescriptionAlreadyExists):
                    create_prescription(
                        subject_identifier=instance.subject_identifier,
                        report_datetime=instance.consent_datetime,
                        medication_names=[instance.study_medication_name],
                        randomizer_name=get_meta_version(),
                        site=instance.site,
                    )

        # create / delete action for reconsent
        if instance.completed_by_next_of_kin == YES:
            ReconsentAction(subject_identifier=instance.subject_identifier)
        else:
            with contextlib.suppress(ActionItemDeleteError):
                delete_action_item(
                    action_cls=ReconsentAction,
                    subject_identifier=instance.subject_identifier,
                )


@receiver(
    post_delete,
    weak=False,
    dispatch_uid="subject_consent_on_post_delete",
)
def subject_consent_on_post_delete(sender, instance, using, **kwargs):  # noqa: ARG001
    """Updates/Resets subject screening."""
    # don't allow if subject visits exist. This should be caught
    # in the ModelAdmin delete view
    if sender in [SubjectConsent, SubjectConsentV1]:
        if SubjectVisit.objects.filter(
            subject_identifier=instance.subject_identifier
        ).exists():
            raise ValidationError("Unable to delete consent. Visit data exists.")

        _, schedule = site_visit_schedules.get_by_onschedule_model("meta_prn.onschedule")
        schedule.take_off_schedule(instance.subject_identifier, instance.consent_datetime)

        # update subject screening
        subject_screening = SubjectScreening.objects.get(
            screening_identifier=instance.screening_identifier
        )
        subject_screening.consented = False
        subject_screening.subject_identifier = subject_screening.subject_screening_as_pk
        subject_screening.save()



# Generated by Django 4.0.5 on 2022-06-27 08:19
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations

#
# def create_missing_offschedule(apps, schema_editor):
#     # get list of missing subjects
#     eos_model_cls = apps.get_model("meta_prn.endofstudy")
#     offschedule_model_cls = apps.get_model("meta_prn.offschedule")
#     action_type_model_cls = apps.get_model("edc_action_item_actiontype")
#     action_type = action_type_model_cls.objects.get(name="offschedule_action")
#
#     for eos in eos_model_cls.objects.all():
#         try:
#             offschedule_model_cls.objects.get(subject_identifier=eos.subject_identifier)
#         except ObjectDoesNotExist:
#             # create action items model instances
#             action_item_model_cls = apps.get_model("edc_action_item_actionitem")
#             try:
#                 action_item_model_cls.objects.get(subject_identifier=eos.subject_identifier, action_type=action_type)
#             except ObjectDoesNotExist:
#
#
#             # create offschedule model instances
#
#
#
#


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0029_alter_historicalpregnancynotification_edd_and_more"),
    ]

    operations = []



from .draw_label_for_subject_with_code128 import draw_label_for_subject_with_code128
from .draw_label_with_test_data import draw_label_with_test_data

__all__ = [
    "draw_label_for_subject_with_code128",
    "draw_label_with_test_data",
]



# Generated by Django 5.0.1 on 2024-02-14 19:39
from django.db import migrations
from django.db.migrations import RunPython
from tqdm import tqdm


def update_refill_end_datetime(apps, schema_editor):
    model_cls = apps.get_model("meta_subject.studymedication")
    qs = model_cls.objects.filter(refill_end_datetime__isnull=True).order_by(
        "subject_visit__subject_identifier"
    )
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0174_remove_glucosefbg_glucose_abnormal_and_more"),
    ]

    operations = [RunPython(update_refill_end_datetime)]
